//-----------------------------------------------------------------------------
#ifndef TLILibImportsH
#define TLILibImportsH TLILibImportsH
//-----------------------------------------------------------------------------
#include <mvIMPACT_CPP/mvIMPACT_acquire.h>

#ifdef __cplusplus
	extern "C" {
#endif

typedef void* MVTLI_HANDLE;
typedef void* MVTLI_INTERFACE_HANDLE;
typedef void* MVTLI_DEVICE_HANDLE;

enum INFO_DATATYPE
{
	INFO_DATATYPE_UNKNOWN = 0,                      ///< Unknown datatype
	INFO_DATATYPE_STRING,                           ///< zero terminated string, the size is including the ending  zero
	INFO_DATATYPE_STRINGLIST,                       ///< concatenated zero terminated strings, ending with an adiitional NULL
	INFO_DATATYPE_INT16,                            ///< singned 16 bit integer
	INFO_DATATYPE_UINT16,                           ///< unsingned 16 bit integer
	INFO_DATATYPE_INT32,                            ///< singned 32 bit integer
	INFO_DATATYPE_UINT32,                           ///< unsingned 32 bit integer
	INFO_DATATYPE_INT64,                            ///< singned 64 bit integer
	INFO_DATATYPE_UINT64,                           ///< unsingned64 bit integer
	INFO_DATATYPE_FLOAT64,                          ///< 64 bit float
	INFO_DATATYPE_PTR,                              ///< void pointer, size system / platform dependent
	INFO_DATATYPE_MAC,                              ///< 6Byte MAC address, binary
	INFO_DATATYPE_BOOL8,                            ///< 8bit bool
	INFO_DATATYPE_SIZET,                            ///< size_t, size system / platform dependent
} ;

enum DEVICE_INFO_CMD
{
	DEVICE_INFO_ID,                                          ///< STRING      String with the ID of the Device
	DEVICE_INFO_MANUFACTURER,                                ///< STRING      String with the Manufacturer of the Device
	DEVICE_INFO_MODEL,                                       ///< STRING      String containing model name
	DEVICE_INFO_TLTYPE,                                      ///< STRING      String describing the standard
	DEVICE_INFO_IP,                                          ///< UINT32_T    IP of the device as uint32_t
	DEVICE_INFO_IP_STRING,                                   ///< STRING      String containing IP of the Device 
	DEVICE_INFO_MAC,                                         ///< STRING      MAC of the Device
	DEVICE_INFO_MAC_STRING,                                  ///< STRING      String containing MAC of the Device 
	DEVICE_INFO_SERIALNUMBER,                                ///< STRING      String containing SN of the Device 
	DEVICE_INFO_ACCESS,                                      ///< UINT32_T    containing GC_DEVICE_ACCESS_FLAGS
	DEVICE_INFO_USERNAME,                                    ///< STRING      String containing the Username of the device
	DEVICE_INFO_DISPLAYNAME,                                 ///< STRING      String containing a display name for the device ( including a unique id )   
	DEVICE_INFO_SUBNET,
	DEVICE_INFO_GATEWAY,
	DEVICE_INFO_SUPPORT_IP_LLA,
	DEVICE_INFO_SUPPORT_IP_DHCP,
	DEVICE_INFO_SUPPORT_IP_PERSISTENT,
	DEVICE_INFO_CURRENT_IP_LLA,
	DEVICE_INFO_CURRENT_IP_DHCP,
	DEVICE_INFO_CURRENT_IP_PERSISTENT,
	DEVICE_INFO_CUSTOM_CMD = 0x10000000,
	DEVICE_INFO_GVCP_MESSAGE_TIMEOUT = DEVICE_INFO_CUSTOM_CMD | 3,
	DEVICE_INFO_SUPPORT_USER_DEFINED_NAME = DEVICE_INFO_CUSTOM_CMD | 6,
	DEVICE_INFO_PERSISTENT_IP_STRING,
	DEVICE_INFO_PERSISTENT_NETMASK_STRING,
	DEVICE_INFO_PERSISTENT_DEFAULT_GATEWAY_STRING,
	DEVICE_INFO_CURRENT_NETMASK_STRING,
	DEVICE_INFO_CURRENT_DEFAULT_GATEWAY_STRING,
	DEVICE_INFO_INTERFACE_COUNT
};

enum DEVICE_ACCESS_FLAGS
{
	DEVICE_ACCESS_UNKNOWN   = 0,         ///< Unknown
	DEVICE_ACCESS_NONE      = 1,         ///< Access denied
	DEVICE_ACCESS_READONLY  = 2,         ///< Host has / can have Read Only access to the device
	DEVICE_ACCESS_CONTROL   = 3,         ///< Host has / can have Control Mode access to the device
	DEVICE_ACCESS_EXCLUSIVE = 4,         ///< Host has / can have Exclusive Mode access to the device
};

int DMR_CALL TLOpen                          ( MVTLI_HANDLE* phTL );
int DMR_CALL TLClose                         ( MVTLI_HANDLE hTL );
int DMR_CALL TLUpdateInterfaceList           ( MVTLI_HANDLE hTL, char* bHasChanged );
int DMR_CALL TLGetNumInterfaces              ( MVTLI_HANDLE hTL, unsigned int* cnt );
int DMR_CALL TLGetInterfaceName              ( MVTLI_HANDLE hTL, unsigned int iIndex, char* psName, size_t* iSize );
int DMR_CALL TLOpenInterface                 ( MVTLI_HANDLE hTL, const char *sInterfaceName, MVTLI_INTERFACE_HANDLE* phInterface );
int DMR_CALL TLCloseInterface                ( MVTLI_INTERFACE_HANDLE hInterface);
int DMR_CALL TLGetNumDevices                 ( MVTLI_INTERFACE_HANDLE hInterface, unsigned int* pNumDevices );
int DMR_CALL TLGetDeviceName                 ( MVTLI_INTERFACE_HANDLE hInterface, unsigned int index, char * psName, size_t* pSize );
int DMR_CALL TLUpdateDeviceList              ( MVTLI_INTERFACE_HANDLE hInterface, unsigned int timeout );
int DMR_CALL TLGetDeviceInfo                 ( MVTLI_INTERFACE_HANDLE hInterface, const char* pDevName, DEVICE_INFO_CMD iID, INFO_DATATYPE* pType, void* pInfo, size_t* pSize );
int DMR_CALL TLOpenDevice                    ( MVTLI_INTERFACE_HANDLE hInterface, const char* pDevName, DEVICE_ACCESS_FLAGS iOpenFlags, MVTLI_DEVICE_HANDLE* pDev );
int DMR_CALL TLIMV_TLGetDeviceInterfaceParam ( MVTLI_INTERFACE_HANDLE hInterface, const char* pDevName, unsigned int interfaceIndex, DEVICE_INFO_CMD iCmd, INFO_DATATYPE *pType, void* pBuf, size_t* pSize );
int DMR_CALL TLIMV_DevSetInterfaceParam      ( MVTLI_DEVICE_HANDLE hDev, unsigned int interfaceIndex, DEVICE_INFO_CMD iCmd, const void* pBuf, size_t bufSize );
int DMR_CALL TLIMV_DevSetParam               ( MVTLI_DEVICE_HANDLE hDev, DEVICE_INFO_CMD iCmd, const void* pBuf, size_t bufSize );
int DMR_CALL DevClose                        ( MVTLI_DEVICE_HANDLE hDev );
int DMR_CALL TLIMV_MACFromSerial             ( const char* pSerial, char* pBuf, size_t* pBufSize );
int DMR_CALL TLIMV_ForceIP                   ( const char* pMACAddress, const char* pNewDeviceIPAddress, const char* pStaticSubnetMask, const char* pStaticDefaultGateway, const char* pAdapterIPAddress, unsigned int timeout_ms );

#ifdef __cplusplus
	}
#endif

#endif // TLILibImportsH
